package edu.sc.seis.cormorant.seismogram;

import java.io.File;
import java.util.Properties;

import org.omg.CORBA.ORB;
import org.omg.PortableServer.Servant;

import edu.sc.seis.cormorant.AbstractController;
import edu.sc.seis.fissuresUtil.cache.BulletproofVestFactory;
import edu.sc.seis.fissuresUtil.cache.ProxySeismogramDC;
import edu.sc.seis.fissuresUtil.namingService.FissuresNamingService;


public class BudLightController extends AbstractController {

    public BudLightController(Properties confProps, String serverPropName, ORB orb) {
        super(confProps, serverPropName, orb);
        File datadir = new File(getProperty("dataDir", "Data"));
        writer = new BudLightWriter(datadir);
        Thread t = new Thread(writer, "budwriter");
        t.setDaemon(true);
        t.start();
        remoteName = getProperty("remoteDC.name");
        remoteDNS = getProperty("remoteDC.dns");
        if (remoteName == null || remoteDNS == null) {
            throw new IllegalArgumentException("Remote server cannot be null: "+remoteDNS+"  "+remoteName);
        }
    }

    @Override
    public void start(ORB orb, FissuresNamingService ns) throws Exception {
        this.ns = ns;
        if (getConfigProps().containsKey(getPropertyKey("seedlink.commands"))) {
            logger.info("Starting seedlink with commands file = "+getProperty("seedlink.commands"));
            seedlink = new BudLightSeedLink(writer.getQueue());
            seedlink.setCommands(BudLightSeedLink.loadCommands(getProperty("seedlink.commands")));
            seedlink.setMaxRecords(-1);
            seedlink.setLocIdFilter(new String[] {"  ", "00"});
            Thread slThread = new Thread(seedlink, "SeedLink");
            slThread.setDaemon(true);
            slThread.start();
        }
        super.start(orb, ns);
    }
    
    @Override
    public void stop(ORB orb, FissuresNamingService ns) throws Exception {
        if (seedlink != null) {
            seedlink.stop();
        }
        super.stop(orb, ns);
        DataRecordQueue q = writer.getQueue();
        synchronized(q) {
            while(q.getSize() > 0) {
                q.notifyAll();
                q.wait();
            }
        }
    }

    @Override
    public Servant getServant() throws Exception {
        if (servant == null) {
            remoteOps = BulletproofVestFactory.vestSeismogramDC(remoteDNS, remoteName, ns, 0); // only do one retry
            servant = new BudLightCachingDC(writer, remoteOps);
        }
        return servant;
    }
    
    String remoteName;
    
    String remoteDNS;
    
    BudLightWriter writer;
    
    FissuresNamingService ns;
    
    ProxySeismogramDC remoteOps; 
    
    BudLightDataCenter servant;
    
    BudLightSeedLink seedlink;
    
    private static final org.slf4j.Logger logger = org.slf4j.LoggerFactory.getLogger(BudLightController.class);
}
